@@ -17,7 +17,7 @@
 (tm-define (mutate-date)
   (:secure #t)
   (with-mutator t
-    (tree-set t (var-eval-system "date +\"%H:%M:%S\""))))
+    (tree-set! t (var-eval-system "date +\"%H:%M:%S\""))))
 
 (tm-define (mutate-blink)
   (:secure #t)
@@ -26,9 +26,9 @@
 	   (s (string->number (var-eval-system "date +\"%S\"")))
 	   (e (mod s 4)))
       (if (and (<= e 1) (not (match? t '(strong :%1))))
-	  (tree-set t `(strong ,t)))
+	  (tree-set! t `(strong ,t)))
       (if (and (>= e 2) (match? t '(strong :%1)))
-	  (tree-set t (tree-ref t 0))))))
+	  (tree-set! t (tree-ref t 0))))))
 
 (kbd-map
   ("C-F11" (insert '(mutator "" "(mutate-date)")))
