@@ -29,6 +29,8 @@
 (define tmtex-packages '())
 (define tmtex-env (make-ahash-table))
 (define tmtex-serial 0)
+(define tmtex-auto-produce 0)
+(define tmtex-auto-consume 0)
 (define tmtex-image-root-url (string->url "image"))
 (define tmtex-image-root-string "image")
 (define tmtex-appendices? #f)
@@ -72,6 +74,8 @@
 (define (tmtex-initialize opts)
   (set! tmtex-env (make-ahash-table))
   (set! tmtex-serial 0)
+  (set! tmtex-auto-produce 0)
+  (set! tmtex-auto-consume 0)
   (if (== (url-suffix current-save-target) "tex")
       (begin
 	(set! tmtex-image-root-url (url-unglue current-save-target 4))
@@ -971,7 +975,7 @@
   (list 'pageref (force-string (car l))))
 
 (define (tmtex-specific l)
-  (cond ((== (car l) "latex") (tmtex (cadr l)))
+  (cond ((== (car l) "latex") (tmtex-tt (cadr l)))
 	((== (car l) "image") (tmtex-eps (cadr l)))
 	(else "")))
 
@@ -986,7 +990,7 @@
   (if (tmtex-math-mode?) (set! x `(with "mode" "math" ,x)))
   (receive (name-url name-string) (tmtex-eps-names)
     (print-snippet name-url x)
-    (list 'epsfig (string-append "file=" name-string))))
+    (list 'includegraphics name-string)))
 
 (define (tmtex-graphics l)
   (tmtex-eps (cons 'graphics l)))
@@ -996,12 +1000,12 @@
     (if (or (string-ends? name ".ps")
 	    (string-ends? name ".eps")
 	    (not (url-exists? u)))
-	(list 'epsfig (string-append "file=" name))
+	(list 'includegraphics name)
 	(let* ((suffix (url-suffix u))
 	       (fm (string-append (format-from-suffix suffix) "-file")))
 	  (receive (name-url name-string) (tmtex-eps-names)
 	    (convert-to-file u fm "postscript-file" name-url)
-	    (list 'epsfig (string-append "file=" name-string)))))))
+	    (list 'includegraphics name-string))))))
 
 (define (tmtex-postscript l)
   (let* ((fig (tmtex-as-eps (force-string (car l))))
@@ -1337,6 +1341,26 @@
 (define (tmtex-cite-year s l)
   (tex-apply 'citeyear (tmtex (car l))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Glossaries
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (tmtex-glossary s l)
+  (with nr (+ tmtex-auto-produce 1)
+    (set! tmtex-auto-produce nr)
+    `(label ,(string-append "autolab" (number->string nr)))))
+
+(define (tmtex-glossary-entry s l)
+  (with nr (+ tmtex-auto-consume 1)
+    (with lab (string-append "autolab" (number->string nr))
+      (set! tmtex-auto-consume nr)
+      `(glossaryentry ,(tmtex (car l)) ,(tmtex (cadr l)) (pageref ,lab)))))
+
+(define (tmtex-the-glossary s l)
+  `(!document
+      (,(if (latex-book-style?) 'chapter* 'section*) "Glossary")
+      ((!begin "theglossary" ,(car l)) ,(tmtex (cadr l)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The main conversion routines
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1544,7 +1568,11 @@
   ((:or equation equation*) (,tmtex-equation-wrapper 1))
   ((:or eqnarray eqnarray* leqnarray*) (,tmtex-eqnarray 1))
   (eq-number (,tmtex-default -1))
-  ((:or the-index the-glossary) (,tmtex-dummy -1))
+  (the-index (,tmtex-dummy -1))
+  (glossary (,tmtex-glossary 1))
+  (glossary-explain (,tmtex-glossary 2))
+  (glossary-2 (,tmtex-glossary-entry 3))
+  (the-glossary (,tmtex-the-glossary 2))
   ((:or table-of-contents) (,tmtex-toc 2))
   (bibliography (,tmtex-bib 4))
   (thebibliography (,tmtex-thebibliography 2))
