@@ -60,6 +60,17 @@
 	 0)))
       0))
 
+(tm-define (points-dist< p1 p2 eps)
+  (set! p1 (frame-direct `(tuple ,(cadr p1) ,(caddr p1))))
+  (set! p2 (frame-direct `(tuple ,(cadr p2) ,(caddr p2))))
+  (set! eps (length-decode eps))
+  (let* ((x1 (s2f (cadr p1)))
+         (y1 (s2f (caddr p1)))
+         (x2 (s2f (cadr p2)))
+         (y2 (s2f (caddr p2)))
+     )
+     (< (+ (* (- x2 x1) (- x2 x1)) (* (- y2 y1) (- y2 y1))) (* eps eps))))
+
 ;; Graphical object
 (tm-define default-color-go-points "#4040ff")
 (tm-define default-color-selected-points "#ff6060")
@@ -74,6 +85,21 @@
 (tm-define graphical-textat-halign "default")
 (tm-define graphical-textat-valign "default")
 
+(tm-define (graphical-fetch-props o)
+  (set! graphical-color (find-prop o "color" "default"))
+  (set! graphical-pstyle (find-prop o "point-style" "default"))
+  (set! graphical-lwidth (find-prop o "line-width" "default"))
+  (set! graphical-magnification (find-prop o "magnification" "default"))
+  (set! graphical-lstyle (find-prop o "dash-style" "default"))
+  (set! graphical-lstyle-unit
+	(find-prop o "dash-style-unit" "default"))
+  (set! graphical-larrows (find-prop o "line-arrows" "default"))
+  (set! graphical-fcolor (find-prop o "fill-color" "default"))
+  (set! graphical-textat-halign
+	(find-prop o "text-at-halign" "default"))
+  (set! graphical-textat-valign
+	(find-prop o "text-at-valign" "default")))
+
 (tm-define (graphical-object fetch)
 ; FIXME: Remove this (tree->stree) should give more speed, but I'm not sure
 ;   about what is the best way now. Then, directly plug the tree and test
@@ -81,20 +107,7 @@
   (with o (tree->stree (get-graphical-object))
    ;(display* "o=" o "\n")
     (if (and fetch (pair? o))
-       (begin
-	  (set! graphical-color (find-prop o "color" "default"))
-	  (set! graphical-pstyle (find-prop o "point-style" "default"))
-	  (set! graphical-lwidth (find-prop o "line-width" "default"))
-	  (set! graphical-magnification (find-prop o "magnification" "default"))
-	  (set! graphical-lstyle (find-prop o "dash-style" "default"))
-	  (set! graphical-lstyle-unit
-		(find-prop o "dash-style-unit" "default"))
-	  (set! graphical-larrows (find-prop o "line-arrows" "default"))
-	  (set! graphical-fcolor (find-prop o "fill-color" "default"))
-	  (set! graphical-textat-halign
-		(find-prop o "text-at-halign" "default"))
-	  (set! graphical-textat-valign
-		(find-prop o "text-at-valign" "default"))))
+	(graphical-fetch-props o))
     (if (pair? o)
 	(if (== (cdr o) '()) o (cAr o))
 	'(concat))))
@@ -526,7 +539,7 @@
 ;; no   == description of the edge  <=> no | (edge no)
 ;;      == flag used in group modes <=> 'group | 'no-group
   (define edge #t)
- ;(display* "o[create-graphical-object]=")(write o)(newline)
+ ;(display* "o[create-graphical-object](" mode "," pts "," no ")=")(write o)(newline)
   (if (pair? no)
       (begin
 	 (set! edge (car no))
@@ -546,17 +559,77 @@
 		)
 	       `(concat .
                   ,(create-graphical-contours
-		      selected-objects current-path pts)
+		      (map (lambda (x)
+			      (if (tree? x)
+				  (enhanced-tree->radical x)
+				  x)
+			   )
+			   the-sketch)
+		      current-path pts)
 		)
 		(append
 		   props
 		  `((concat . ,(cond ((== pts 'points) op)
 				     ((== pts 'object) `(,o))
 				     ((== pts 'object-and-points)
-				      (cons o op))))))))
-      )
+				      (cons o op)))))))))
       (graphical-object! '(concat))))
 
+(tm-define (graphics-decorations-update . parms)
+;; Creating the graphical object exclusively from the context
+  (invalidate-graphical-object)
+  (if (== (length parms) 4)
+      (create-graphical-object
+	 (car parms) (cadr parms) (caddr parms) (cadddr parms))
+      (if (graphics-group-mode? (graphics-mode))
+	  (with pts #f
+	     (if (null? parms)
+		 (set! pts (if sticky-point 'object 'points))
+		 (set! pts (car parms)))
+	     (if (and (null? (sketch-get))
+		      (pair? current-obj)
+		      (> (length current-obj) 1))
+		 (create-graphical-object current-obj '() 'points 'no-group)
+		 (create-graphical-object current-obj current-path pts #f)))
+	  (let* ((mode (if (null? parms)
+			   (if sticky-point
+			      'active
+			       current-path)
+			   (car parms)))
+		 (tag (if current-obj (car current-obj) #f))
+	     )
+	     (if (and sticky-point
+		      (== 1 (length (sketch-get))))
+		 (set! current-obj (car (sketch-get))))
+	     (if (tree? current-obj)
+		 (set! current-obj (tree->stree current-obj)))
+	     (if (and (eq? mode 'active)
+		      (pair? current-obj))
+		 (begin
+		    (graphical-fetch-props 
+		       (if (eq? (car current-obj) 'with)
+			   current-obj `(with ,current-obj)))
+		    (set! current-obj (stree-radical current-obj))))
+	     (create-graphical-object
+		current-obj
+		mode
+		(if sticky-point 'object-and-points 'points)
+                (if (== tag 'text-at)
+                    1
+                    (cond ((or (not tag)
+			       (== tag 'gr-group))
+                           #f)
+                          (else
+			     (if current-point-no
+				 (if current-edge-sel?
+				     current-point-no
+				    `(,current-edge-sel? ,current-point-no))
+				 #f))))))))
+  (invalidate-graphical-object))
+
+(tm-define (graphics-decorations-reset)
+  (create-graphical-object #f #f #f #f))
+
 ;; Operating on the graphical object
 (tm-define (transform-graphical-object opn)
   (with o (tree->stree (get-graphical-object))
@@ -564,3 +637,95 @@
      (begin
 	(set! o (opn o))
 	(graphical-object! o)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Managing the sketch
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (get-root x)
+  (if (and (tree? x) (tree->path x))
+      (radical->enhanced-tree x)
+      x))
+
+(tm-define (sketch-get)
+  the-sketch)
+
+(tm-define (sketch-set! l)
+  (set! the-sketch (map get-root l)))
+
+(tm-define (sketch-in? t)
+  (set! t (get-root t))
+  (seek-eq? t the-sketch))
+
+(tm-define (sketch-reset)
+  (set! the-sketch '()))
+
+(tm-define (sketch-toggle t)
+  (set! t (get-root t))
+  (set! the-sketch
+        (if (sketch-in? t)
+            (remove-eq? t the-sketch)
+            (rcons the-sketch t))))
+
+(tm-define (sketch-transform opn)
+  (set! the-sketch (map opn the-sketch))
+  (set! current-obj
+	(if (graphics-group-mode? (graphics-mode)) '(nothing) #f))
+  (set! current-path #f)
+  (graphics-decorations-update))
+
+;; TODO: Replace sticky-point by a more appropriate name for
+;;   telling whether we are in SELECTING or MODIFYING mode.
+(tm-define (sketch-checkout)
+  (if sticky-point
+      (graphics-error "(sketch-checkout)"))
+  (set! layer-of-last-removed-object
+	(if (== (length (sketch-get)) 1)
+	    #f
+	   '()))
+  (set! remove-undo-mark? #f)
+  (foreach-cons (c (sketch-get))
+     (with o (car c)
+     (if (tree? o)
+     (begin
+	(set! remove-undo-mark? #t)
+	(graphics-remove (tree->path o) 'memoize-layer)
+	(if (or (tree->path o) (tree->path (enhanced-tree->radical o)))
+	    (set-car! c (tree-copy o))))))
+  )
+  (set! sticky-point #t)
+  (set! graphics-undo-enabled #f)
+  (graphics-decorations-update))
+
+(tm-define (sketch-commit)
+  (define group-mode? (graphics-group-mode? (graphics-mode)))
+  (if (not sticky-point)
+      (graphics-error "(sketch-commit)"))
+  (with sketch0 (list-copy (sketch-get))
+     (sketch-reset)
+     (foreach (o sketch0)
+	(with layer layer-of-last-removed-object
+	   (sketch-toggle (path->tree (graphics-group-insert-bis o group-mode?)))
+	   (if (not (list? layer))
+	       (set! layer-of-last-removed-object layer)))
+     )
+     (set! layer-of-last-removed-object #f)
+     (set! current-obj
+	   (if group-mode?
+	      '(nothing)
+	       (if (== 1 (length (sketch-get)))
+		   (stree-radical (tm->stree (car (sketch-get))))
+		   #f)))
+     (set! current-path
+	   (if (and (== 1 (length (sketch-get)))
+		    (tree? (car (sketch-get))))
+	       (tree->path (car (sketch-get)))))
+  )
+  (set! sticky-point #f)
+  (set! graphics-undo-enabled #t)
+  (if remove-undo-mark?
+      (remove-undo-mark))
+  (graphics-decorations-update))
+
+(tm-define (sketch-cancel)
+  #t)
