@@ -28,10 +28,10 @@
      )
      (export ,(car head))))
      (export define-export-macro)
-  ;; NOTE: Deprecated. It seems that as soon as macros become a little bit
-  ;;   complex, the Guile macroexpander interacts poorly with the memoizing
-  ;;   stuff in (define-public-macro), and then it becomes unstable. This
-  ;;   is the reason why (define-export-macro) exists : it is *strictly*
+  ;; NOTE: It seems that as soon as macros become a little bit complex,
+  ;;   the Guile macroexpander interacts poorly with the memoizing stuff
+  ;;   in (define-public-macro), and then it becomes unstable. This is
+  ;;   the reason why (define-export-macro) exists : it is *strictly*
   ;;   equivalent to (define-macro) + (export), and never raises problems.
 
 ;; Conversions
@@ -64,9 +64,49 @@
 (tm-define t2o tree->object)
 (tm-define o2t object->tree)
 
+;; Error
+(tm-define (graphics-error . msg)
+  (foreach (e msg)
+     (display e))
+  (newline) ;(quit-TeXmacs)
+)
+
 ;; Lists as bags
-(tm-define seek-eq? memq)
-(tm-define remove-eq? delq1!)
+;; FIXME: One more time, due to an incomplete implementation
+;;   of some very basic functionnality, we had to do our own
+;;   hack...
+;(tm-define seek-eq? memq)
+;(tm-define remove-eq? delq1!)
+
+(tm-define (complete-eq? x y)
+  (if (and (tree? x) (tree? y))
+      (tree-eq? x y)
+      (eq? x y)))
+
+(tm-define (seek-eq? x l)
+  (if (pair? l)
+      (if (complete-eq? x (car l))
+          l
+          (seek-eq? x (cdr l)))
+      #f))
+
+(tm-define (remove-eq0? x l)
+  (define l0 (cons 'X l))
+  (define (seek l prec)
+     (if (pair? l)
+         (if (complete-eq? x (car l))
+             (set-cdr! prec (cdr l))
+             (seek (cdr l) (cdr prec))))
+  )
+  (seek l l0)
+  (cdr l0))
+
+(define-export-macro (remove-eq? x l)
+  (if (symbol? l)
+     `(begin
+	 (set! ,l (remove-eq0? ,x ,l))
+	,l)
+     `(remove-eq0? ,x ,l)))
 
 ;; Iterators
 (define-public foreach for)
@@ -171,7 +211,7 @@
       (begin
 	 (set-car! list-find-prop-cons val)
          l)
-     `(with ,var ,val . ,l)))
+     `(with ,var ,val . ,(if (eq? (car l) 'with) (cdr l) l))))
 
 (tm-define (tm-find-prop p var)
 ;;(tm-find-prop '<tree <with|a|1|...>> "a")              -> <tree 1>
@@ -247,8 +287,8 @@
 (tm-define (graphics-graphics-path)
   ;; path to innermost graphics tag
   (let* ((p (cDr (cursor-path)))
-	 (t (stree-at p)))
-    (if (func? t 'graphics) p
+	 (t (path->tree p)))
+    (if (eq? (tree-label t) 'graphics) p
 	(with u (tree-innermost 'graphics)
 	  (and u (tree->path u))))))
 
@@ -279,7 +319,7 @@
    '(with "gr-mode" "point" 
           "gr-frame" (tuple "scale" "1cm" (tuple "0.5gw" "0.5gh"))
           "gr-geometry" (tuple "geometry" "1par" "0.6par")
-      (graphics))
+      (graphics ""))
    '(6 1)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -481,7 +521,7 @@
                              val)
 			  layer-of-last-removed-object)
                       (tree-arity (path->tree p)))
-	    (path-insert (rcons p n) (list 'tuple t))
+	    (tree-insert (path->tree p) n (list t))
 	    (if (func? t 'with)
 		(if (and go-into (func? (cAr t) 'text-at))
 		    (set! p2 (append p (list n (- (length t) 2) 0 0)))
@@ -509,7 +549,7 @@
 (tm-define (graphics-group-start)
   (graphics-finish)
   (with p (graphics-group-path)
-    (if p (go-to (rcons p 0)))))
+    (if p (go-to (append p '(0 0))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Subroutines for modifying the active tag
@@ -579,7 +619,7 @@
 (tm-define (graphics-active-assign t)
   (with p (graphics-active-path)
     (if p (begin
-	    (path-assign p t)
+	    (tree-assign (path->tree p) t)
 	    (go-to (rcons p 1))))))
 
 (tm-define (graphics-active-set-tag l)
@@ -589,7 +629,7 @@
 (tm-define (graphics-active-insert t)
   (with p (graphics-active-path)
     (if p (with n (tree-arity (path->tree p))
-	    (path-insert (rcons p n) (list 'tuple t))
+	    (tree-insert (path->tree p) n (list t))
 	    (go-to (rcons p 1))))))
 
 (tm-define (graphics-object-root-path p)
@@ -601,6 +641,7 @@
 	path))
     
 (tm-define (graphics-remove p . parms)
+  (if p
   (with p0 (graphics-object-root-path p)
      (set! layer-of-last-removed-object
 	   (if (and (pair? parms) (eq? (car parms) 'memoize-layer))
@@ -608,7 +649,12 @@
 		   (cons (cAr p0) layer-of-last-removed-object)
 		   (cAr p0))
 	       #f))
-     (path-remove p0 1)))
+     (tree-remove (path->tree (cDr p0)) (cAr p0) 1))))
+
+(tm-define (graphics-assign p t)
+  (if p (begin
+	  (tree-assign (path->tree p) t)
+	  (go-to (rcons p 1)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Box info & frame
@@ -654,3 +700,66 @@
 			      `(,(car box2) ,(caddr box2)))
 	 (interval-intersects `(,(cadr box1) ,(cadddr box1))
 			      `(,(cadr box2) ,(cadddr box2))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Enhanced trees
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (enhanced-tree? t)
+  (eq? (tree-label t) 'with))
+
+(tm-define (enhanced-tree->radical t)
+  (if (enhanced-tree? t)
+      (tree-ref t (- (tree-arity t) 1))
+      t))
+
+(tm-define (stree-radical t)
+  (if (and (pair? t) (eq? (car t) 'with) (nnull? (cdr t)))
+      (cAr t)
+      t))
+
+(tm-define (radical->enhanced-tree r)
+  (with t (tree-up r)
+     (if (enhanced-tree? t) t r)))
+
+(tm-define (enhanced-tree-arity t)
+  (tree-arity (enhanced-tree->radical t)))
+
+(tm-define (enhanced-tree-ref t i)
+  (tree-ref (enhanced-tree->radical t) i))
+
+(tm-define (enhanced-tree-set! t i val)
+  (tree-set! (enhanced-tree->radical t) i val))
+
+(tm-define (enhanced-tree-insert! t i val)
+  (tree-insert! (enhanced-tree->radical t) i `(,val)))
+
+(tm-define (enhanced-tree-remove! t i n)
+  (tree-remove! (enhanced-tree->radical t) i n))
+
+(tm-define (enhanced-tree-property-ref t var)
+  (define res #f)
+  (if (string? var)
+      (set! var (stree->tree var)))
+  (if (enhanced-tree? t)
+      (foreach-number (i 0 < (- (tree-arity r) 1))
+	 (if (equal? (tree-ref t i) var)
+	     (set! res (tree-ref t (+ i 1))))
+	 (set! i (+ 1 i))))
+  res)
+
+(tm-define (enhanced-tree-property-set! t var val)
+  (define found #f)
+  (if (string? var)
+      (set! var (stree->tree var)))
+  (if (enhanced-tree? t)
+      (begin
+	 (foreach-number (i 0 < (- (tree-arity r) 1))
+	    (if (equal? (tree-ref t i) var)
+		(begin
+		   (set! found #t)
+		   (tree-set! t (+ i 1) val)))
+	    (set! i (+ 1 i))
+	 )
+         (if (not found)
+	     (tree-insert! t (tree-arity t) `(,var ,val))))))
