@@ -601,11 +601,11 @@ latex_parser::parse_verbatim (string s, int& i, string end) {
 ******************************************************************************/
 
 static char Cork_unaccented[128]= {
-  'A', ' ', 'C', 'C', 'D', 'E', ' ', 'G',
+  'A', 'A', 'C', 'C', 'D', 'E', 'E', 'G',
   'L', 'L', ' ', 'N', 'N', ' ', 'O', 'R',
   'R', 'S', 'S', 'S', 'T', 'T', 'U', 'U',
   'Y', 'Z', 'Z', 'Z', ' ', 'I', 'd', ' ',
-  'a', ' ', 'c', 'c', 'd', 'e', ' ', 'g',
+  'a', 'a', 'c', 'c', 'd', 'e', 'e', 'g',
   'l', 'l', ' ', 'n', 'n', ' ', 'o', 'r',
   'r', 's', 's', 's', 't', 't', 'u', 'u',
   'y', 'z', 'z', 'z', ' ', ' ', ' ', ' ',
@@ -620,13 +620,13 @@ static char Cork_unaccented[128]= {
 };
 
 static char Cork_accent[128]= {
-  'u' , ' ' , '\'', 'v' , 'v' , 'v' , ' ' , 'u' ,
-  '\'', ' ' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
-  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , ' ' ,
+  'u' , 'k' , '\'', 'v' , 'v' , 'v' , 'k' , 'u' ,
+  '\'', 'v' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
+  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , 'r' ,
   '\"', '\'', 'v' , '.' , ' ' , '.' , '=' , ' ' , // "
-  'u' , ' ' , '\'', 'v' , 'v' , 'v' , ' ' , 'u' ,
-  '\'', ' ' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
-  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , ' ' ,
+  'u' , 'k' , '\'', 'v' , 'v' , 'v' , 'k' , 'u' ,
+  '\'', 'v' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
+  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , 'r' ,
   '\"', '\'', 'v' , '.' , ' ' , ' ' , ' ' , ' ' , // "
   '`' , '\'', '^' , '~' , '\"', ' ' , ' ' , 'c' , // "
   '`' , '\'', '^' , '\"', '`' , '\'', '^' , '\"', // "
