@@ -18,6 +18,8 @@
 #include "convert.hpp"
 #include "file.hpp"
 #include "sys_utils.hpp"
+#include "Tmfs/tmfs.hpp"
+#include "client_server.hpp"
 #include "analyze.hpp"
 #include "tree_traverse.hpp"
 #include "tm_layout.hpp"
@@ -320,9 +322,10 @@ tree_ref (tree t, int i) {
   return t[i];
 }
 
-void
+tree
 tree_set (tree t, int i, tree u) {
   t[i]= u;
+  return u;
 }
 
 tree
@@ -335,6 +338,132 @@ tree_append (tree t1, tree t2) {
   return t1 * t2;
 }
 
+bool
+tree_active (tree t) {
+  path ip= obtain_ip (t);
+  return nil (ip) || last_item (ip) != DETACHED;
+}
+
+tree
+tree_child_insert (tree t, int pos, tree x) {
+  cout << "t= " << t << "\n";
+  cout << "x= " << x << "\n";
+  int i, n= N(t);
+  tree r (t, n+1);
+  for (i=0; i<pos; i++) r[i]= t[i];
+  r[pos]= x;
+  for (i=pos; i<n; i++) r[i+1]= t[i];
+  return r;
+}
+
+/******************************************************************************
+* Document modification routines
+******************************************************************************/
+
+static inline bool
+ip_attached (path ip) {
+  return nil (ip) || last_item (ip) != DETACHED;
+}
+
+tree
+tree_assign (tree r, tree t) {
+  path ip= obtain_ip (r);
+  //cout << "Assign " << r << ", " << t << " at " << ip << "\n";
+  if (ip_attached (ip)) {
+    path p= reverse (ip);
+    get_server()->get_editor()->assign (p, copy (t));
+    return get_server()->get_editor()->the_subtree (p);
+  }
+  else {
+    assign (r, t);
+    return r;
+  }
+}
+
+tree
+tree_insert (tree r, int pos, tree t) {
+  path ip= obtain_ip (r);
+  //cout << "Insert " << r << ", " << pos << ", " << t
+  //     << " at " << ip << "\n";
+  if (ip_attached (ip))
+    get_server()->get_editor()->insert (reverse (ip) * pos, copy (t));
+  else insert (r, pos, t);
+  return r;
+}
+
+tree
+tree_remove (tree r, int pos, int nr) {
+  path ip= obtain_ip (r);
+  //cout << "Remove " << r << ", " << pos << ", " << nr
+  //     << " at " << ip << "\n";
+  if (ip_attached (ip))
+    get_server()->get_editor()->remove (reverse (ip) * pos, nr);
+  else remove (r, pos, nr);
+  return r;
+}
+
+tree
+tree_split (tree r, int pos, int at) {
+  path ip= obtain_ip (r);
+  //cout << "Split " << r << ", " << pos << ", " << at
+  //     << " at " << ip << "\n";
+  if (ip_attached (ip))
+    get_server()->get_editor()->split (reverse (ip) * path (pos, at));
+  else split (r, pos, at);
+  return r;
+}
+
+tree
+tree_join (tree r, int pos) {
+  path ip= obtain_ip (r);
+  //cout << "Join " << r << ", " << pos << " at " << ip << "\n";
+  if (ip_attached (ip))
+    get_server()->get_editor()->join (reverse (ip) * pos);
+  else join (r, pos);
+  return r;
+}
+
+tree
+tree_assign_node (tree r, tree_label op) {
+  path ip= obtain_ip (r);
+  //cout << "Assign node " << r << ", " << tree (op) << " at " << ip << "\n";
+  if (ip_attached (ip))
+    get_server()->get_editor()->assign_node (reverse (ip), op);
+  else assign_node (r, op);
+  return r;
+}
+
+tree
+tree_insert_node (tree r, int pos, tree t) {
+  path ip= obtain_ip (r);
+  //cout << "Insert node " << r << ", " << pos << ", " << t
+  //     << " at " << ip << "\n";
+  if (ip_attached (ip)) {
+    path p= reverse (ip);
+    get_server()->get_editor()->insert_node (p * pos, copy (t));
+    return get_server()->get_editor()->the_subtree (p);
+  }
+  else {
+    insert_node (r, pos, t);
+    return r;
+  }
+}
+
+tree
+tree_remove_node (tree r, int pos) {
+  path ip= obtain_ip (r);
+  //cout << "Remove node " << r << ", " << pos << " at " << ip << "\n";
+  if (ip_attached (ip)) {
+    path p= reverse (ip);
+    get_server()->get_editor()->remove_node (p * pos);
+    return get_server()->get_editor()->the_subtree (p);
+  }
+  else {
+    remove_node (r, pos);
+    return r;
+  }
+}
+
 /******************************************************************************
 * Scheme trees
 ******************************************************************************/
@@ -386,15 +515,28 @@ scm_to_scheme_tree (SCM p) {
 * Content
 ******************************************************************************/
 
+bool
+scm_is_content (SCM p) {
+  if (scm_is_string (p) || scm_is_tree (p)) return true;
+  else if (!scm_is_pair (p) || !scm_is_symbol (SCM_CAR (p))) return false;
+  else {
+    for (p= SCM_CDR (p); !scm_is_null (p); p= SCM_CDR (p))
+      if (!scm_is_content (SCM_CAR (p))) return false;
+    return true;
+  }
+}
+
 #define content tree
-#define SCM_ASSERT_CONTENT(p,arg,rout)
+#define SCM_ASSERT_CONTENT(p,arg,rout) \
+  SCM_ASSERT (scm_is_content (p), p, arg, rout)
 #define content_to_scm tree_to_scm
 
 tree
 scm_to_content (SCM p) {
+  if (scm_is_string (p)) return scm_to_string (p);
   if (scm_is_tree (p)) return scm_to_tree (p);
-  if (scm_is_list (p)) {
-    if (scm_is_null (p) || (!scm_is_symbol (SCM_CAR (p)))) return "?";
+  if (scm_is_pair (p)) {
+    if (!scm_is_symbol (SCM_CAR (p))) return "?";
     tree t (make_tree_label (scm_to_symbol (SCM_CAR (p))));
     p= SCM_CDR (p);
     while (!scm_is_null (p)) {
@@ -403,13 +545,15 @@ scm_to_content (SCM p) {
     }
     return t;
   }
-  if (scm_is_symbol (p)) return scm_to_symbol (p);
-  if (scm_is_string (p)) return scm_to_string (p);
-  if (scm_is_int (p)) return as_string ((int) scm_to_int (p));
-  if (scm_is_bool (p)) return (scm_to_bool (p)? string ("#t"): string ("#f"));
   return "?";
 }
 
+static SCM
+contentP (SCM t) {
+  bool b= scm_is_content (t);
+  return bool_to_scm (b);
+}
+
 /******************************************************************************
 * Paths
 ******************************************************************************/
@@ -761,9 +905,61 @@ urlP (SCM t) {
 url url_concat (url u1, url u2) { return u1 * u2; }
 url url_or (url u1, url u2) { return u1 | u2; }
 void string_save (string s, url u) { (void) save_string (u, s); }
-string string_load (url u) { string s; (void) load_string (u, s); return s; }
+string string_load (url u) {
+  string s; (void) load_string (u, s, false); return s; }
 url url_ref (url u, int i) { return u[i]; }
 
+/******************************************************************************
+* Table types
+******************************************************************************/
+
+typedef hashmap<string,string> table_string_string;
+
+static bool
+scm_is_table_string_string (SCM p) {
+  if (scm_is_null (p)) return true;
+  else if (!scm_is_pair (p)) return false;
+  else {
+    SCM f= SCM_CAR (p);
+    return scm_is_pair (f) &&
+           scm_is_string (SCM_CAR (f)) &&
+           scm_is_string (SCM_CDR (f)) &&
+           scm_is_table_string_string (SCM_CDR (p));
+  }
+}
+
+#define SCM_ASSERT_TABLE_STRING_STRING(p,arg,rout) \
+  SCM_ASSERT (scm_is_table_string_string (p), p, arg, rout)
+
+SCM
+table_string_string_to_scm (hashmap<string,string> t) {
+  SCM p= SCM_NULL;
+  iterator<string> it= iterate (t);
+  while (it->busy ()) {
+    string s= it->next ();
+    SCM n= scm_cons (string_to_scm (s), string_to_scm (t[s]));
+    p= scm_cons (n, p);
+  }
+  return p;
+}
+
+hashmap<string,string>
+scm_to_table_string_string (SCM p) {
+  hashmap<string,string> t;
+  while (!scm_is_null (p)) {
+    SCM n= SCM_CAR (p);
+    t (scm_to_string (SCM_CAR (n)))= scm_to_string (SCM_CDR (n));
+    p= SCM_CDR (p);
+  }
+  return t;
+}
+
+#define scm_is_solution scm_is_table_string_string
+#define SCM_ASSERT_SOLUTION(p,arg,rout) \
+  SCM_ASSERT (scm_is_solution(p), p, arg, rout)
+#define solution_to_scm table_string_string_to_scm
+#define scm_to_solution scm_to_table_string_string
+
 /******************************************************************************
 * Several array types
 ******************************************************************************/
@@ -776,7 +972,9 @@ typedef array<widget> array_widget;
 static bool
 scm_is_array_int (SCM p) {
   if (scm_is_null (p)) return true;
-  else return scm_is_int (SCM_CAR (p)) && scm_is_array_int (SCM_CDR (p));
+  else return scm_is_pair (p) &&
+	      scm_is_int (SCM_CAR (p)) &&
+	      scm_is_array_int (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_ARRAY_INT(p,arg,rout) \
@@ -803,7 +1001,9 @@ scm_to_array_int (SCM p) {
 static bool
 scm_is_array_string (SCM p) {
   if (scm_is_null (p)) return true;
-  else return scm_is_string (SCM_CAR (p)) && scm_is_array_string (SCM_CDR (p));
+  else return scm_is_pair (p) && 
+	      scm_is_string (SCM_CAR (p)) &&
+	      scm_is_array_string (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_ARRAY_STRING(p,arg,rout) \
@@ -827,10 +1027,17 @@ scm_to_array_string (SCM p) {
   return a;
 }
 
+#define scm_is_property scm_is_array_string
+#define SCM_ASSERT_PROPERTY(p,arg,rout) SCM_ASSERT_ARRAY_STRING (p,arg,rout)
+#define property_to_scm array_string_to_scm
+#define scm_to_property scm_to_array_string
+
 static bool
 scm_is_array_tree (SCM p) {
   if (scm_is_null (p)) return true;
-  else return SCM_TREEP (SCM_CAR (p)) && scm_is_array_tree (SCM_CDR (p));
+  else return scm_is_pair (p) && 
+	      SCM_TREEP (SCM_CAR (p)) &&
+	      scm_is_array_tree (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_ARRAY_TREE(p,arg,rout) \
@@ -857,7 +1064,9 @@ scm_to_array_tree (SCM p) {
 static bool
 scm_is_array_widget (SCM p) {
   if (scm_is_null (p)) return true;
-  else return scm_is_widget (SCM_CAR (p)) && scm_is_array_widget (SCM_CDR (p));
+  else return scm_is_pair (p) &&
+	      scm_is_widget (SCM_CAR (p)) &&
+	      scm_is_array_widget (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_ARRAY_WIDGET(p,arg,rout) \
@@ -881,6 +1090,64 @@ scm_to_array_widget (SCM p) {
   return a;
 }
 
+static bool
+scm_is_properties (SCM p) {
+  if (scm_is_null (p)) return true;
+  else return scm_is_pair (p) &&
+	      scm_is_property (SCM_CAR (p)) &&
+	      scm_is_properties (SCM_CDR (p));
+}
+
+#define SCM_ASSERT_PROPERTIES(p,arg,rout) \
+  SCM_ASSERT (scm_is_properties (p), p, arg, rout)
+
+SCM
+properties_to_scm (array<property> a) {
+  int i, n= N(a);
+  SCM p= SCM_NULL;
+  for (i=n-1; i>=0; i--) p= scm_cons (property_to_scm (a[i]), p);
+  return p;
+}
+
+array<property>
+scm_to_properties (SCM p) {
+  array<property> a;
+  while (!scm_is_null (p)) {
+    a << scm_to_property (SCM_CAR (p));
+    p= SCM_CDR (p);
+  }
+  return a;
+}
+
+static bool
+scm_is_solutions (SCM p) {
+  if (scm_is_null (p)) return true;
+  else return scm_is_pair (p) &&
+	      scm_is_solution (SCM_CAR (p)) &&
+	      scm_is_solutions (SCM_CDR (p));
+}
+
+#define SCM_ASSERT_SOLUTIONS(p,arg,rout) \
+  SCM_ASSERT (scm_is_solutions (p), p, arg, rout)
+
+SCM
+solutions_to_scm (array<solution> a) {
+  int i, n= N(a);
+  SCM p= SCM_NULL;
+  for (i=n-1; i>=0; i--) p= scm_cons (solution_to_scm (a[i]), p);
+  return p;
+}
+
+array<solution>
+scm_to_solutions (SCM p) {
+  array<solution> a;
+  while (!scm_is_null (p)) {
+    a << scm_to_solution (SCM_CAR (p));
+    p= SCM_CDR (p);
+  }
+  return a;
+}
+
 /******************************************************************************
 * List types
 ******************************************************************************/
@@ -890,7 +1157,9 @@ typedef list<string> list_string;
 bool
 scm_is_list_string (SCM p) {
   if (scm_is_null (p)) return true;
-  else return scm_is_string (SCM_CAR (p)) && scm_is_list_string (SCM_CDR (p));
+  else return scm_is_pair (p) &&
+	      scm_is_string (SCM_CAR (p)) &&
+	      scm_is_list_string (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_LIST_STRING(p,arg,rout) \
@@ -915,7 +1184,9 @@ typedef list<tree> list_tree;
 bool
 scm_is_list_tree (SCM p) {
   if (scm_is_null (p)) return true;
-  else return scm_is_tree (SCM_CAR (p)) && scm_is_list_tree (SCM_CDR (p));
+  else return scm_is_pair (p) &&
+	      scm_is_tree (SCM_CAR (p)) &&
+	      scm_is_list_tree (SCM_CDR (p));
 }
 
 #define SCM_ASSERT_LIST_TREE(p,arg,rout) \
@@ -935,6 +1206,23 @@ scm_to_list_tree (SCM p) {
 		    scm_to_list_tree (SCM_CDR (p)));
 }
 
+/******************************************************************************
+* Other wrapper types
+******************************************************************************/
+
+#define SCM_ASSERT_COLLECTION(p,arg,rout) \
+  SCM_ASSERT (scm_is_array_string (p), p, arg, rout)
+
+SCM
+collection_to_scm (collection ss) {
+  return array_string_to_scm (as_strings (ss));
+}
+
+collection
+scm_to_collection (SCM p) {
+  return as_collection (scm_to_array_string (p));
+}
+
 /******************************************************************************
 * Initialization
 ******************************************************************************/
@@ -979,6 +1267,7 @@ initialize_glue () {
   scm_set_smob_print (url_tag, print_url);
   scm_set_smob_equalp (url_tag, cmp_url);
   scm_new_procedure ("tree?", (FN) treeP, 1, 0, 0);
+  scm_new_procedure ("tm?", (FN) contentP, 1, 0, 0);
   scm_new_procedure ("observer?", (FN) observerP, 1, 0, 0);
   scm_new_procedure ("url?", (FN) urlP, 1, 0, 0);
   initialize_glue_basic ();
@@ -1026,6 +1315,7 @@ initialize_glue () {
   command_tag= scm_newsmob (&command_smob_funcs);
   url_tag= scm_newsmob (&url_smob_funcs);
   scm_new_procedure ("tree?", (FN) treeP, 1, 0, 0);
+  scm_new_procedure ("tm?", (FN) contentP, 1, 0, 0);
   scm_new_procedure ("observer?", (FN) observerP, 1, 0, 0);
   scm_new_procedure ("url?", (FN) urlP, 1, 0, 0);
   initialize_glue_basic ();
