@@ -23,10 +23,19 @@ extern text_property_rep tp_operator_rep;
 extern text_property_rep tp_shortop_rep;
 
 struct verb_language_rep: language_rep {
-  verb_language_rep ();
+  verb_language_rep (string name);
   text_property advance (string s, int& pos);
   array<int> get_hyphens (string s);
   void hyphenate (string s, int after, string& left, string& right);
 };
 
+struct scheme_language_rep: language_rep {
+  hashmap<string,string> colored;
+  scheme_language_rep (string name);
+  text_property advance (string s, int& pos);
+  array<int> get_hyphens (string s);
+  void hyphenate (string s, int after, string& left, string& right);
+  string get_color (tree t, int start, int end);
+};
+
 #endif // defined IMPL_LANGUAGE_H
