@@ -14,6 +14,7 @@
 #include "convert.hpp"
 #include "file.hpp"
 #include "web_files.hpp"
+#include "socket_server.hpp"
 #ifdef EXPERIMENTAL
 #include "../../Style/Memorizer/clean_copy.hpp"
 #endif
@@ -54,6 +55,15 @@ tm_data_rep::find_buffer (url name) {
 string
 tm_data_rep::new_menu_name (url u) {
   string name= as_string (tail (u));
+  if (starts (name, "no_name_") && ends (name, ".tm")) {
+    display dis= current_display ();
+    string lan= dis->get_output_language ();
+    string no_name= translate ("No name", "english", lan);
+    for (int i=0; i<N(no_name); i++)
+      if (((unsigned char) (no_name[i])) >= (unsigned char) 128)
+	{ no_name= "No name"; break; }
+    name= no_name * " [" * name (8, N(name) - 3) * "]";
+  }
   if ((name == "") || (name == "."))
     name= as_string (tail (u * url_parent ()));
   if ((name == "") || (name == "."))
@@ -257,6 +267,14 @@ tm_data_rep::get_name_buffer (path p) {
   return bufs[nr]->name;
 }
 
+url
+tm_data_rep::get_all_buffers () {
+  url u= url_none ();
+  for (int i=N(bufs)-1; i>=0; i--)
+    u= bufs[i]->name | u;
+  return u;
+}
+
 /******************************************************************************
 * Low level view routines
 ******************************************************************************/
@@ -428,15 +446,17 @@ tm_data_rep::get_buf (path p) {
   else return NULL;
 }
 
-void
+url
 tm_data_rep::new_buffer () {
-  int i;
-  for (i=1; true; i++) {
-    string name= "no name " * as_string (i);
-    if (i==1) name= "no name";
-    if (find_buffer (name) != -1) continue;
-    new_buffer_in_this_window (name, tree (DOCUMENT));
-    return;
+  int i=1;
+  while (true) {
+    url name= url_scratch ("no_name_", ".tm", i);
+    int nr= find_buffer (name);
+    if (nr == -1) {
+      new_buffer_in_this_window (name, tree (DOCUMENT));
+      return name;
+    }
+    else i++;
   }
 }
 
@@ -532,15 +552,17 @@ tm_data_rep::kill_buffer () {
   delete_buffer (buf);
 }
 
-void
+url
 tm_data_rep::open_window (tree geom) {
-  int i;
-  for (i=1; true; i++) {
-    string name= "no name " * as_string (i);
-    if (i==1) name= "no name";
-    if (find_buffer (name) != -1) continue;
-    new_buffer_in_new_window (name, tree (DOCUMENT), geom);
-    return;
+  int i=1;
+  while (true) {
+    url name= url_scratch ("no_name_", ".tm", i);
+    int nr= find_buffer (name);
+    if (nr == -1) {
+      new_buffer_in_new_window (name, tree (DOCUMENT), geom);
+      return name;
+    }
+    else i++;
   }
 }
 
@@ -570,7 +592,8 @@ tm_data_rep::kill_window () {
       }
     }
   }
-  quit ();
+  if (number_of_servers () == 0) quit ();
+  else delete_window (win);
 }
 
 void
@@ -768,6 +791,96 @@ set_document (path rp, tree t) {
 #endif
 }
 
+/******************************************************************************
+* Window management
+******************************************************************************/
+
+static int  last_window= 1;
+static path the_windows;
+
+int
+create_window_id () {
+  the_windows= path (last_window, the_windows);
+  return last_window++;
+}
+
+static path
+remove (path p, int i) {
+  if (nil (p)) return p;
+  else if (p->item == i) return p->next;
+  else return path (p->item, remove (p->next, i));
+}
+
+void
+destroy_window_id (int i) {
+  the_windows= remove (the_windows, i);
+}
+
+int
+tm_data_rep::window_current () {
+  tm_window win= get_window ();
+  return win->id;
+}
+
+path
+tm_data_rep::windows_list () {
+  return the_windows;
+}
+
+path
+tm_data_rep::buffer_to_windows (url name) {
+  path p;
+  int nr= find_buffer (name);
+  if (nr == -1) return path ();
+  tm_buffer buf= bufs[nr];
+  for (int i=0; i<N(buf->vws); i++)
+    if (buf->vws[i]->win != NULL)
+      p= path (buf->vws[i]->win->id, p);
+  return p;
+}
+
+url
+tm_data_rep::window_to_buffer (int id) {
+  for (int i=0; i<N(bufs); i++)
+    for (int j=0; j<N(bufs[i]->vws); j++)
+      if (bufs[i]->vws[j]->win != NULL)
+	if (bufs[i]->vws[j]->win->id == id)
+	  return bufs[i]->name;
+  return url_none ();
+}
+
+tm_view
+tm_data_rep::window_find_view (int id) {
+  for (int i=0; i<N(bufs); i++)
+    for (int j=0; j<N(bufs[i]->vws); j++)
+      if (bufs[i]->vws[j]->win != NULL)
+	if (bufs[i]->vws[j]->win->id == id)
+	  return bufs[i]->vws[j];
+  return NULL;
+}
+
+void
+tm_data_rep::window_set_buffer (int id, url name) {
+  tm_view old_vw= window_find_view (id);
+  if (old_vw == NULL || old_vw->buf->name == name) return;
+  tm_window win= old_vw->win;
+  int nr= find_buffer (name);
+  if (nr == -1) return;
+  tm_buffer buf   = bufs[nr];
+  tm_view   new_vw= get_passive_view (buf);
+  detach_view (old_vw);
+  attach_view (win, new_vw);
+}
+
+void
+tm_data_rep::window_focus (int id) {
+  if (id == window_current ()) return;
+  tm_view vw= window_find_view (id);
+  if (vw == NULL) return;
+  set_view (vw);
+  menu_focus_buffer (vw->buf);
+}
+
 /******************************************************************************
 * Undo/redo routines for buffers as well as tests for (auto)saves
 ******************************************************************************/
